@@ -49,6 +49,21 @@ def regression_set(regression_Xy) -> [pd.DataFrame, np.ndarray]:
     return pd.DataFrame(regression_Xy[0]), regression_Xy[1]
 
 
+@pytest.fixture
+def regression_hasna_set() -> [pd.DataFrame, np.ndarray]:
+    N = 1000
+    dim = 10
+
+    x = np.random.uniform(0, 1, size=(N, dim))
+    x = np.where(x < .3, np.inf, x)
+    x = np.where(x < .5, np.nan, x)
+    y = np.random.uniform(size=N)
+
+    df = pd.DataFrame(x)
+
+    return df, y
+
+
 @pytest.fixture
 def toy_df() -> pd.DataFrame:
     data = {
